@@ -70,31 +70,9 @@ static int                nni_aio_expire_q_cnt;
 // operations from starting, without waiting for any existing one to
 // complete, call nni_aio_close.
 
-// In some places we want to check that an aio is not in use.
-// Technically if these checks pass, then they should not need
-// to be done with a lock, because the caller should have the only
-// references to them.  However, race detectors can't necessarily
-// know about this semantic, and may complain about potential data
-// races.  To suppress false positives, define NNG_RACE_DETECTOR.
-// Note that this will cause extra locks to be acquired, affecting
-// performance, so don't use it in production.
-#ifdef __has_feature
-#if __has_feature(thread_sanitizer)
-#define NNG_RACE_DETECTOR
-#endif
-#endif
-
-#ifdef NNG_RACE_DETECTOR
-#define aio_safe_lock(l) nni_mtx_lock(l)
-#define aio_safe_unlock(l) nni_mtx_unlock(l)
-#else
-#define aio_safe_lock(l) ((void) 1)
-#define aio_safe_unlock(l) ((void) 1)
-#endif
-
 static nni_reap_list aio_reap_list = {
 	.rl_offset = offsetof(nni_aio, a_reap_node),
-	.rl_func   = (nni_cb) nni_aio_free,
+	.rl_func   = nni_aio_free_cb,
 };
 
 static void nni_aio_expire_add(nni_aio *);
@@ -136,6 +114,8 @@ nni_aio_fini(nni_aio *aio)
 
 	if (fn != NULL) {
 		fn(aio, arg, NNG_ECLOSED);
+	} else {
+		nni_task_abort(&aio->a_task);
 	}
 
 	nni_task_fini(&aio->a_task);
@@ -163,6 +143,12 @@ nni_aio_free(nni_aio *aio)
 	}
 }
 
+void
+nni_aio_free_cb(void *aio)
+{
+	nni_aio_free((nni_aio *) aio);
+}
+
 void
 nni_aio_reap(nni_aio *aio)
 {
@@ -215,6 +201,8 @@ nni_aio_stop(nni_aio *aio)
 
 		if (fn != NULL) {
 			fn(aio, arg, NNG_ECANCELED);
+		} else {
+			nni_task_abort(&aio->a_task);
 		}
 
 		nni_aio_wait(aio);
@@ -240,6 +228,8 @@ nni_aio_close(nni_aio *aio)
 
 		if (fn != NULL) {
 			fn(aio, arg, NNG_ECLOSED);
+		} else {
+			nni_task_abort(&aio->a_task);
 		}
 	}
 }
@@ -344,8 +334,7 @@ nni_aio_begin(nni_aio *aio)
 	// checks may wish ignore or suppress these checks.
 	nni_aio_expire_q *eq = aio->a_expire_q;
 
-	aio_safe_lock(&eq->eq_mtx);
-
+	nni_mtx_lock(&eq->eq_mtx);
 	NNI_ASSERT(!nni_aio_list_active(aio));
 	NNI_ASSERT(aio->a_cancel_fn == NULL);
 	NNI_ASSERT(!nni_list_node_active(&aio->a_expire_node));
@@ -359,9 +348,6 @@ nni_aio_begin(nni_aio *aio)
 	aio->a_count     = 0;
 	aio->a_cancel_fn = NULL;
 
-	aio_safe_unlock(&eq->eq_mtx);
-
-	nni_mtx_lock(&eq->eq_mtx);
 	// We should not reschedule anything at this point.
 	if (aio->a_stop) {
 		aio->a_result    = NNG_ECANCELED;
@@ -439,6 +425,8 @@ nni_aio_abort(nni_aio *aio, int rv)
 	// Stop any I/O at the provider level.
 	if (fn != NULL) {
 		fn(aio, arg, rv);
+	} else {
+		nni_task_abort(&aio->a_task);
 	}
 }
 
