@@ -1,5 +1,5 @@
 //
-// Copyright 2018 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2025 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 // Copyright 2018 Devolutions <info@devolutions.net>
 //
@@ -12,8 +12,6 @@
 #ifndef CORE_LISTENER_H
 #define CORE_LISTENER_H
 
-extern int      nni_listener_sys_init(void);
-extern void     nni_listener_sys_fini(void);
 extern int      nni_listener_find(nni_listener **, uint32_t);
 extern int      nni_listener_hold(nni_listener *);
 extern void     nni_listener_rele(nni_listener *);
@@ -27,6 +25,7 @@ extern int nni_listener_setopt(
     nni_listener *, const char *, const void *, size_t, nni_type);
 extern int nni_listener_getopt(
     nni_listener *, const char *, void *, size_t *, nni_type);
+extern const nng_url *nni_listener_url(nni_listener *);
 extern void nni_listener_add_stat(nni_listener *, nni_stat_item *);
 extern void nni_listener_bump_error(nni_listener *, int);
 
