@@ -22,16 +22,16 @@ typedef struct ipc_conn {
 	HANDLE        f;
 	nni_win_io    recv_io;
 	nni_win_io    send_io;
-	nni_win_io    conn_io;
 	nni_list      recv_aios;
 	nni_list      send_aios;
-	nni_aio      *conn_aio;
 	nng_sockaddr  sa;
 	bool          dialer;
 	int           recv_rv;
 	int           send_rv;
 	int           conn_rv;
 	bool          closed;
+	bool          sending;
+	bool          recving;
 	nni_mtx       mtx;
 	nni_cv        cv;
 	nni_reap_node reap;
@@ -48,48 +48,48 @@ ipc_recv_start(ipc_conn *c)
 	DWORD    len;
 	int      rv;
 
-	if (c->closed) {
-		while ((aio = nni_list_first(&c->recv_aios)) != NULL) {
-			nni_list_remove(&c->recv_aios, aio);
+	while ((aio = nni_list_first(&c->recv_aios)) != NULL) {
+		if (c->closed) {
+			nni_aio_list_remove(aio);
 			nni_aio_finish_error(aio, NNG_ECLOSED);
+			continue;
 		}
-		nni_cv_wake(&c->cv);
-	}
-again:
-	if ((aio = nni_list_first(&c->recv_aios)) == NULL) {
-		return;
-	}
 
-	nni_aio_get_iov(aio, &naiov, &aiov);
+		nni_aio_get_iov(aio, &naiov, &aiov);
 
-	idx = 0;
-	while ((idx < naiov) && (aiov[idx].iov_len == 0)) {
-		idx++;
-	}
-	NNI_ASSERT(idx < naiov);
-	// Now start a transfer.  We assume that only one send can be
-	// outstanding on a pipe at a time.  This is important to avoid
-	// scrambling the data anyway.  Note that Windows named pipes do
-	// not appear to support scatter/gather, so we have to process
-	// each element in turn.
-	buf = aiov[idx].iov_buf;
-	len = (DWORD) aiov[idx].iov_len;
-	NNI_ASSERT(buf != NULL);
-	NNI_ASSERT(len != 0);
+		idx = 0;
+		while ((idx < naiov) && (aiov[idx].iov_len == 0)) {
+			idx++;
+		}
+		NNI_ASSERT(idx < naiov);
+		// Now start a transfer.  We assume that only one send can be
+		// outstanding on a pipe at a time.  This is important to avoid
+		// scrambling the data anyway.  Note that Windows named pipes
+		// do not appear to support scatter/gather, so we have to
+		// process each element in turn.
+		buf = aiov[idx].iov_buf;
+		len = (DWORD) aiov[idx].iov_len;
+		NNI_ASSERT(buf != NULL);
+		NNI_ASSERT(len != 0);
 
-	// We limit ourselves to writing 16MB at a time.  Named Pipes
-	// on Windows have limits of between 31 and 64MB.
-	if (len > 0x1000000) {
-		len = 0x1000000;
-	}
+		// We limit ourselves to writing 16MB at a time.  Named Pipes
+		// on Windows have limits of between 31 and 64MB.
+		if (len > 0x1000000) {
+			len = 0x1000000;
+		}
 
-	if ((!ReadFile(c->f, buf, len, NULL, &c->recv_io.olpd)) &&
-	    ((rv = GetLastError()) != ERROR_IO_PENDING)) {
-		// Synchronous failure.
-		nni_aio_list_remove(aio);
-		nni_aio_finish_error(aio, nni_win_error(rv));
-		goto again;
+		c->recving = true;
+		if ((!ReadFile(c->f, buf, len, NULL, &c->recv_io.olpd)) &&
+		    ((rv = GetLastError()) != ERROR_IO_PENDING)) {
+			// Synchronous failure.
+			c->recving = false;
+			nni_aio_list_remove(aio);
+			nni_aio_finish_error(aio, nni_win_error(rv));
+		} else {
+			return;
+		}
 	}
+	nni_cv_wake(&c->cv);
 }
 
 static void
@@ -98,11 +98,8 @@ ipc_recv_cb(nni_win_io *io, int rv, size_t num)
 	nni_aio  *aio;
 	ipc_conn *c = io->ptr;
 	nni_mtx_lock(&c->mtx);
-	if ((aio = nni_list_first(&c->recv_aios)) == NULL) {
-		// Should indicate that it was closed.
-		nni_mtx_unlock(&c->mtx);
-		return;
-	}
+	aio = nni_list_first(&c->recv_aios);
+	NNI_ASSERT(aio != NULL);
 	if (c->recv_rv != 0) {
 		rv         = c->recv_rv;
 		c->recv_rv = 0;
@@ -111,12 +108,9 @@ ipc_recv_cb(nni_win_io *io, int rv, size_t num)
 		// A zero byte receive is a remote close from the peer.
 		rv = NNG_ECONNSHUT;
 	}
+	c->recving = false;
 	nni_aio_list_remove(aio);
-	if (c->closed) {
-		nni_cv_wake(&c->cv);
-	} else {
-		ipc_recv_start(c);
-	}
+	ipc_recv_start(c);
 	nni_mtx_unlock(&c->mtx);
 
 	nni_aio_finish_sync(aio, rv, num);
@@ -130,10 +124,16 @@ ipc_recv_cancel(nni_aio *aio, void *arg, int rv)
 	if (aio == nni_list_first(&c->recv_aios)) {
 		c->recv_rv = rv;
 		CancelIoEx(c->f, &c->recv_io.olpd);
-	} else if (nni_aio_list_active(aio)) {
-		nni_aio_list_remove(aio);
-		nni_aio_finish_error(aio, rv);
-		nni_cv_wake(&c->cv);
+	} else {
+		nni_aio *srch;
+		NNI_LIST_FOREACH (&c->recv_aios, srch) {
+			if (srch == aio) {
+				nni_aio_list_remove(aio);
+				nni_aio_finish_error(aio, rv);
+				nni_cv_wake(&c->cv);
+				break;
+			}
+		}
 	}
 	nni_mtx_unlock(&c->mtx);
 }
@@ -176,48 +176,43 @@ ipc_send_start(ipc_conn *c)
 	DWORD    len;
 	int      rv;
 
-	if (c->closed) {
-		while ((aio = nni_list_first(&c->send_aios)) != NULL) {
-			nni_list_remove(&c->send_aios, aio);
-			nni_aio_finish_error(aio, NNG_ECLOSED);
+	while ((aio = nni_list_first(&c->send_aios)) != NULL) {
+
+		nni_aio_get_iov(aio, &naiov, &aiov);
+
+		idx = 0;
+		while ((idx < naiov) && (aiov[idx].iov_len == 0)) {
+			idx++;
 		}
-		nni_cv_wake(&c->cv);
-	}
-again:
-	if ((aio = nni_list_first(&c->send_aios)) == NULL) {
-		return;
-	}
+		NNI_ASSERT(idx < naiov);
+		// Now start a transfer.  We assume that only one send can be
+		// outstanding on a pipe at a time.  This is important to avoid
+		// scrambling the data anyway.  Note that Windows named pipes
+		// do not appear to support scatter/gather, so we have to
+		// process each element in turn.
+		buf = aiov[idx].iov_buf;
+		len = (DWORD) aiov[idx].iov_len;
+		NNI_ASSERT(buf != NULL);
+		NNI_ASSERT(len != 0);
 
-	nni_aio_get_iov(aio, &naiov, &aiov);
+		// We limit ourselves to writing 16MB at a time.  Named Pipes
+		// on Windows have limits of between 31 and 64MB.
+		if (len > 0x1000000) {
+			len = 0x1000000;
+		}
 
-	idx = 0;
-	while ((idx < naiov) && (aiov[idx].iov_len == 0)) {
-		idx++;
-	}
-	NNI_ASSERT(idx < naiov);
-	// Now start a transfer.  We assume that only one send can be
-	// outstanding on a pipe at a time.  This is important to avoid
-	// scrambling the data anyway.  Note that Windows named pipes do
-	// not appear to support scatter/gather, so we have to process
-	// each element in turn.
-	buf = aiov[idx].iov_buf;
-	len = (DWORD) aiov[idx].iov_len;
-	NNI_ASSERT(buf != NULL);
-	NNI_ASSERT(len != 0);
-
-	// We limit ourselves to writing 16MB at a time.  Named Pipes
-	// on Windows have limits of between 31 and 64MB.
-	if (len > 0x1000000) {
-		len = 0x1000000;
-	}
-
-	if ((!WriteFile(c->f, buf, len, NULL, &c->send_io.olpd)) &&
-	    ((rv = GetLastError()) != ERROR_IO_PENDING)) {
-		// Synchronous failure.
-		nni_aio_list_remove(aio);
-		nni_aio_finish_error(aio, nni_win_error(rv));
-		goto again;
+		c->sending = true;
+		if ((!WriteFile(c->f, buf, len, NULL, &c->send_io.olpd)) &&
+		    ((rv = GetLastError()) != ERROR_IO_PENDING)) {
+			// Synchronous failure.
+			c->sending = false;
+			nni_aio_list_remove(aio);
+			nni_aio_finish_error(aio, nni_win_error(rv));
+		} else {
+			return;
+		}
 	}
+	nni_cv_wake(&c->cv);
 }
 
 static void
@@ -226,20 +221,15 @@ ipc_send_cb(nni_win_io *io, int rv, size_t num)
 	nni_aio  *aio;
 	ipc_conn *c = io->ptr;
 	nni_mtx_lock(&c->mtx);
-	if ((aio = nni_list_first(&c->send_aios)) == NULL) {
-		// Should indicate that it was closed.
-		nni_mtx_unlock(&c->mtx);
-		return;
-	}
+	aio = nni_list_first(&c->send_aios);
+	NNI_ASSERT(aio != NULL);
+	nni_aio_list_remove(aio);
+	c->sending = false;
 	if (c->send_rv != 0) {
 		rv         = c->send_rv;
 		c->send_rv = 0;
 	}
-	nni_aio_list_remove(aio);
 	ipc_send_start(c);
-	if (c->closed) {
-		nni_cv_wake(&c->cv);
-	}
 	nni_mtx_unlock(&c->mtx);
 
 	nni_aio_finish_sync(aio, rv, num);
@@ -253,10 +243,16 @@ ipc_send_cancel(nni_aio *aio, void *arg, int rv)
 	if (aio == nni_list_first(&c->send_aios)) {
 		c->send_rv = rv;
 		CancelIoEx(c->f, &c->send_io.olpd);
-	} else if (nni_aio_list_active(aio)) {
-		nni_aio_list_remove(aio);
-		nni_aio_finish_error(aio, rv);
-		nni_cv_wake(&c->cv);
+	} else {
+		nni_aio *srch;
+		NNI_LIST_FOREACH (&c->recv_aios, srch) {
+			if (srch == aio) {
+				nni_aio_list_remove(aio);
+				nni_aio_finish_error(aio, rv);
+				nni_cv_wake(&c->cv);
+				break;
+			}
+		}
 	}
 	nni_mtx_unlock(&c->mtx);
 }
@@ -271,11 +267,6 @@ ipc_send(void *arg, nni_aio *aio)
 		return;
 	}
 	nni_mtx_lock(&c->mtx);
-	if (c->closed) {
-		nni_mtx_unlock(&c->mtx);
-		nni_aio_finish_error(aio, NNG_ECLOSED);
-		return;
-	}
 	if ((rv = nni_aio_schedule(aio, ipc_send_cancel, c)) != 0) {
 		nni_mtx_unlock(&c->mtx);
 		nni_aio_finish_error(aio, rv);
@@ -292,16 +283,32 @@ static void
 ipc_close(void *arg)
 {
 	ipc_conn *c = arg;
+	nni_time  now;
 	nni_mtx_lock(&c->mtx);
 	if (!c->closed) {
+		HANDLE f  = c->f;
 		c->closed = true;
-		CancelIoEx(c->f, NULL); // cancel all requests
 
-		if (c->f != INVALID_HANDLE_VALUE) {
-			// NB: closing the pipe is dangerous at this point.
-			DisconnectNamedPipe(c->f);
+		c->f = INVALID_HANDLE_VALUE;
+
+		if (f != INVALID_HANDLE_VALUE) {
+			CancelIoEx(f, &c->send_io.olpd);
+			CancelIoEx(f, &c->recv_io.olpd);
+			DisconnectNamedPipe(f);
+			CloseHandle(f);
 		}
 	}
+	now = nni_clock();
+	// wait up to a maximum of 10 seconds before assuming something is
+	// badly amiss. from what we can tell, this doesn't happen, and we do
+	// see the timer expire properly, but this safeguard can prevent a
+	// hang.
+	while ((c->recving || c->sending) &&
+	    ((nni_clock() - now) < (NNI_SECOND * 10))) {
+		nni_mtx_unlock(&c->mtx);
+		nni_msleep(1);
+		nni_mtx_lock(&c->mtx);
+	}
 	nni_mtx_unlock(&c->mtx);
 }
 
@@ -317,10 +324,6 @@ ipc_conn_reap(void *arg)
 	}
 	nni_mtx_unlock(&c->mtx);
 
-	nni_win_io_fini(&c->recv_io);
-	nni_win_io_fini(&c->send_io);
-	nni_win_io_fini(&c->conn_io);
-
 	if (c->f != INVALID_HANDLE_VALUE) {
 		CloseHandle(c->f);
 	}
@@ -403,7 +406,6 @@ nni_win_ipc_init(
     nng_stream **connp, HANDLE p, const nng_sockaddr *sa, bool dialer)
 {
 	ipc_conn *c;
-	int       rv;
 
 	if ((c = NNI_ALLOC_STRUCT(c)) == NULL) {
 		return (NNG_ENOMEM);
@@ -422,11 +424,8 @@ nni_win_ipc_init(
 	c->stream.s_get   = ipc_get;
 	c->stream.s_set   = ipc_set;
 
-	if (((rv = nni_win_io_init(&c->recv_io, ipc_recv_cb, c)) != 0) ||
-	    ((rv = nni_win_io_init(&c->send_io, ipc_send_cb, c)) != 0)) {
-		ipc_free(c);
-		return (rv);
-	}
+	nni_win_io_init(&c->recv_io, ipc_recv_cb, c);
+	nni_win_io_init(&c->send_io, ipc_send_cb, c);
 
 	c->f   = p;
 	*connp = (void *) c;
