@@ -334,6 +334,10 @@ resp0_pipe_close(void *arg)
 
 	nni_mtx_lock(&s->mtx);
 	p->closed = true;
+	if (nni_list_active(&s->recvpipes, p)) {
+		// We are no longer "receivable".
+		nni_list_remove(&s->recvpipes, p);
+	}
 	while ((ctx = nni_list_first(&p->sendq)) != NULL) {
 		nni_aio *aio;
 		nni_msg *msg;
