@@ -1,5 +1,5 @@
 //
-// Copyright 2021 Staysail Systems, Inc. <info@staystail.tech>
+// Copyright 2022 Staysail Systems, Inc. <info@staystail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 //
 // This software is supplied under the terms of the MIT License, a
@@ -53,7 +53,7 @@ check_props_v6(nng_msg *msg)
 	z = sizeof(nng_sockaddr);
 	p = nng_msg_get_pipe(msg);
 	So(nng_pipe_id(p) > 0);
-	So(nng_pipe_getopt(p, NNG_OPT_LOCADDR, &la, &z) == 0);
+	So(nng_pipe_get(p, NNG_OPT_LOCADDR, &la, &z) == 0);
 	So(z == sizeof(la));
 	So(la.s_family == NNG_AF_INET6);
 	// So(la.s_in.sa_port == (trantest_port - 1));
@@ -65,7 +65,7 @@ check_props_v6(nng_msg *msg)
 	z = sizeof(nng_sockaddr);
 	p = nng_msg_get_pipe(msg);
 	So(nng_pipe_id(p) > 0);
-	So(nng_pipe_getopt(p, NNG_OPT_REMADDR, &ra, &z) == 0);
+	So(nng_pipe_get(p, NNG_OPT_REMADDR, &ra, &z) == 0);
 	So(z == sizeof(ra));
 	So(ra.s_family == NNG_AF_INET6);
 	So(ra.s_in6.sa_port != 0);
@@ -78,7 +78,7 @@ TestMain("TCP (IPv6) Transport", {
 	nni_init();
 
 	if (has_v6()) {
-		trantest_test_extended("tcp://[::1]:%u", check_props_v6);
+		trantest_test_extended("tcp://[::1]:", check_props_v6);
 	} else {
 		SkipSo("IPv6 not available");
 	}
