@@ -1,5 +1,5 @@
 //
-// Copyright 2021 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2022 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 //
 // This software is supplied under the terms of the MIT License, a
@@ -143,24 +143,24 @@ check_props(nng_msg *msg)
 	So(nng_pipe_id(p) > 0);
 
 	z = sizeof(nng_sockaddr);
-	So(nng_pipe_getopt(p, NNG_OPT_LOCADDR, &la, &z) == 0);
+	So(nng_pipe_get(p, NNG_OPT_LOCADDR, &la, &z) == 0);
 	So(z == sizeof(la));
 	So(validloopback(&la));
 
 	z = sizeof(nng_sockaddr);
-	So(nng_pipe_getopt(p, NNG_OPT_REMADDR, &ra, &z) == 0);
+	So(nng_pipe_get(p, NNG_OPT_REMADDR, &ra, &z) == 0);
 	So(z == sizeof(ra));
 	So(validloopback(&ra));
 
 	// Request header
 	z   = 0;
 	buf = NULL;
-	So(nng_pipe_getopt(p, NNG_OPT_WS_REQUEST_HEADERS, buf, &z) ==
+	So(nng_pipe_get(p, NNG_OPT_WS_REQUEST_HEADERS, buf, &z) ==
 	    NNG_EINVAL);
 	So(z > 0);
 	len = z;
 	So((buf = nng_alloc(len)) != NULL);
-	So(nng_pipe_getopt(p, NNG_OPT_WS_REQUEST_HEADERS, buf, &z) == 0);
+	So(nng_pipe_get(p, NNG_OPT_WS_REQUEST_HEADERS, buf, &z) == 0);
 	So(strstr(buf, "Sec-WebSocket-Key") != NULL);
 	So(z == len);
 	nng_free(buf, len);
@@ -168,12 +168,12 @@ check_props(nng_msg *msg)
 	// Response header
 	z   = 0;
 	buf = NULL;
-	So(nng_pipe_getopt(p, NNG_OPT_WS_RESPONSE_HEADERS, buf, &z) ==
+	So(nng_pipe_get(p, NNG_OPT_WS_RESPONSE_HEADERS, buf, &z) ==
 	    NNG_EINVAL);
 	So(z > 0);
 	len = z;
 	So((buf = nng_alloc(len)) != NULL);
-	So(nng_pipe_getopt(p, NNG_OPT_WS_RESPONSE_HEADERS, buf, &z) == 0);
+	So(nng_pipe_get(p, NNG_OPT_WS_RESPONSE_HEADERS, buf, &z) == 0);
 	So(strstr(buf, "Sec-WebSocket-Accept") != NULL);
 	So(z == len);
 	nng_free(buf, len);
@@ -200,7 +200,7 @@ init_dialer_wss(nng_dialer d)
 	    0) {
 		goto out;
 	}
-	rv = nng_dialer_setopt_ptr(d, NNG_OPT_TLS_CONFIG, cfg);
+	rv = nng_dialer_set_ptr(d, NNG_OPT_TLS_CONFIG, cfg);
 
 out:
 	nng_tls_config_free(cfg);
@@ -220,7 +220,7 @@ init_listener_wss(nng_listener l)
 		goto out;
 	}
 
-	if ((rv = nng_listener_setopt_ptr(l, NNG_OPT_TLS_CONFIG, cfg)) != 0) {
+	if ((rv = nng_listener_set_ptr(l, NNG_OPT_TLS_CONFIG, cfg)) != 0) {
 		// We can wind up with EBUSY from the server already running.
 		if (rv == NNG_EBUSY) {
 			rv = 0;
@@ -241,7 +241,7 @@ TestMain("WebSocket Secure (TLS) Transport", {
 
 	tt.dialer_init   = init_dialer_wss;
 	tt.listener_init = init_listener_wss;
-	tt.tmpl          = "wss://localhost:%u/test";
+	tt.tmpl          = "wss://localhost:";
 	tt.proptest      = check_props;
 
 	trantest_test(&tt);
